/*
Main.c: top level routines
*/
#include "io.h"
#include "ad.h"
#include "report.h"
#include "idletimer.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "usbdrv/usbdrv.h"

uchar ReportBuffer[REPORT_BUFFER_LEN];
uchar OldReportBuffer[REPORT_BUFFER_LEN];

uchar IdleRate;

/*
Control packet handler. This routine returns a report (or gets/sets the idlerate)
when the PC explicitly requests it without the uC having given it an interrupt.
*/
uchar usbFunctionSetup(uchar data[8]) {
    usbRequest_t *rq=(void *)data;

    if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) {    /* class request type */
        if (rq->bRequest == USBRQ_HID_GET_REPORT) {  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            usbMsgPtr=ReportBuffer;
	    //return not more than requested
            if (rq->wLength.word<REPORT_BUFFER_LEN) {
        	return rq->wLength.word;
            } else {
        	return REPORT_BUFFER_LEN;
    	    }
        } else if(rq->bRequest == USBRQ_HID_GET_IDLE) {
            usbMsgPtr = &IdleRate;
            return 1;
        } else if(rq->bRequest == USBRQ_HID_SET_IDLE) {
            IdleRate = rq->wValue.bytes[1];
        }
    } else {
        /* no vendor specific requests implemented */
    }
    return 0;
}

/*
Main routine. Here the magic happens :)
*/
int main(void) {
    int x;
    int p,off;
    char changed;
    
    IdleRate=250;

    //init hardware
    IoInit();
    AdInit();
    IdleTimerInit();
    usbInit();
    sei();
    
    //Fill the reports with some sensible data
    BuildReport(OldReportBuffer);
    BuildReport(ReportBuffer);

    while(1) {
	//handle usb stuff
	usbPoll();

	//Build a report from the analog channels and digital buttons.
	BuildReport(ReportBuffer);

	//Compare the buikt report to an old one.
	changed=0;
	for (x=0; x<REPORT_BUFFER_LEN; x++) {
	    if (ReportBuffer[x]!=OldReportBuffer[x]) changed=1;
	}
	//If changed=1, the report needs to be sent to the PC.
	//If the idle-timer has timed out, it needs to be sent to the PC too.
	if (IdleTimerGet()>IdleRate) changed=1;
	
	//If needed and possible, send the report.
	if(changed && usbInterruptIsReady()) {
	    //First, make this report the most current.
	    for (x=0; x<REPORT_BUFFER_LEN; x++) {
		OldReportBuffer[x]=ReportBuffer[x];
	    }

	    //Now send it.

	    //usbSetInterrupt only sends 8 bytes at a time, so we have to hack 
	    //around it to send all bytes in the report.
	    //See http://forums.obdev.at/viewtopic.php?=&p=364 for more
	    //info. usbFunctionSetup doesn't have that restriction. by the way.

	    usbSetInterrupt(ReportBuffer, REPORT_BUFFER_LEN>8?8:REPORT_BUFFER_LEN);
	    p=REPORT_BUFFER_LEN-8;
	    off=8;
	    while(p>0) {
		while(!usbInterruptIsReady()) usbPoll();
		usbSetInterrupt(ReportBuffer+off, p>7?8:p);
		p-=8;
		off+=8;
	    }
	    //Reset the idle timer, because we sent some data to the PC.
	    IdleTimerReset();
	}
    }
    return 0;
}

